﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using ProjectBase.Tools.Logging;
using ProjectBase.Tools.Logging.Wrapper;
using System.Threading;

namespace ClientMM
{
    // Program sending 1000 times a request to the MultiMonitoring server
    // and logging the elpased time between sending of the request and receiving 
    // of the response.
    class Program
    {
        private readonly static ILogger ThomasLogger = ProjectBase.Logging.LoggerHelper.GetLogger("Thomas.Logging");

        static void Main()
        {
            RunAll();
        }

        static void RunAll()
        {
            // Sends 1000 times the query
            int count = 1000;

            for (int i = 0; i < count; i++)
            {
                RunAsync().Wait();
                Thread.Sleep(100);
            }
        }

        static async Task RunAsync()
        {
            using (var client = new HttpClient())
            {
                client.BaseAddress = new Uri("http://localhost:9018/");
                client.DefaultRequestHeaders.Accept.Clear();
                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                
                // Starts measuring
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                // OData
                HttpResponseMessage response = await client.GetAsync("odata/Jobs?$expand=Destinations,JobMonitor&$count=false&$skip=0&$top=20&$orderby=MeanFrameRate desc, Id desc");
                

                // Cache
                
                // All jobs
                //HttpResponseMessage response = await client.GetAsync("multimonitoring/jobs");

                // Top 20
                //HttpResponseMessage response = await client.GetAsync("multimonitoring/jobs/?_offset=0&_max=20&_sort=");

                // Top 20 and Skip 80
                //HttpResponseMessage response = await client.GetAsync("multimonitoring/jobs/?_offset=80&_max=20&_sort=Id:asc");

                // Filter
                //HttpResponseMessage response = await client.GetAsync("multimonitoring/jobs/?_offset=0&_max=20&_sort=&ShortStatus=Failed");

                // Sort
                //HttpResponseMessage response = await client.GetAsync("multimonitoring/jobs/?_offset=0&_max=20&_sort=StartTime:asc");


                // Stops measuring
                stopwatch.Stop();
                
                // Logs the measured time
                Console.WriteLine("Time elapsed: {0}", stopwatch.Elapsed);

                if (response.IsSuccessStatusCode)
                {
                    // Fetch ID
                    string id = response.Headers.GetValues("x-tracking-id").FirstOrDefault();

                    // Logs ID and measured time
                    ThomasLogger.LogInfo("Client " + id + " " + stopwatch.Elapsed);
                }
                else
                {
                    ThomasLogger.LogInfo("Client-ERROR ");
                }
            }
        }
    }
}
